#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import shutil

from ezsub import const
from ezsub.cache import Cache
from ezsub.utils import to_screen
from ezsub.destination import Destination


cur = const.Curser


def backup(req):
    # does not need '.zip' extension
    file_name = f"{const.PROGRAMNAME}-{const.TODAY_TIMEADD}"
    destination = Destination(req.destination, req.group, req.open_after)
    to_file = destination.get_child(file_name, mk_parents=True)
    cache = Cache()
    try:
        to_screen(f"Backup {const.PROGRAMNAME} cache directory to:")
        to_screen(f"    {to_file.resolve()}.zip", style="info")
        to_screen("started...", end='')
        shutil.make_archive(to_file, 'zip', root_dir=cache.root, base_dir=cache.root)
        to_screen(f"\r{cur.CL}Done.", style="ok")
        destination.open()
    except KeyboardInterrupt:
        out_file = to_file.with_suffix('.zip')
        if out_file.exists():
            to_screen(f"\nRemoving incomplete backup file '{out_file.name}' from destination...", style="warn")
            out_file.unlink()
            to_screen("Done!", style="ok")
        raise KeyboardInterrupt
    return None



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from ezsub.cli.commands.backup import backup
from ezsub.cli.commands.clean import clean
from ezsub.cli.commands.config import config
from ezsub.cli.commands.download import download
from ezsub.cli.commands.history import history
from ezsub.cli.commands.info import info
from ezsub.cli.commands.login import login
from ezsub.cli.commands.unzip import unzip
from ezsub.cli.commands.update import update



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from ezsub.cache import Cache, prune
from ezsub.destination import Destination
from ezsub.utils import select, parse_lngs
from ezsub.errors import NoResultError


def unzip(req):
    destination = Destination(req.destination, req.group, req.open_after)
    cache = Cache()
    if req.exact:
        results, selected = cache.exact_search(req.exact)
    else:  # use title
        results = cache.search(req.title)
        selected = select(results, req.auto_select)

    if not selected:
        raise NoResultError

    paths = [results[s-1]['path'] for s in selected]
    lngs = parse_lngs(req.lngs)
    to_extract = prune(paths, lngs)
    destination.extract(to_extract)
    return None



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from ezsub.cache import Cache, prune
from ezsub.errors import NoResultError, NothingToCleanError
from ezsub.utils import to_screen, select, parse_lngs, get_size, human_readable


def clean(req):
    cache = Cache()
    if req.all:
        to_screen("Just for your information:", style='blue;bold')
        to_screen("You can make a back-up from your current cache by calling:", style='warn')
        to_screen("\tezsub backup -o", style='ok')
        if not req.zero:
            to_screen("Also it is recommended to use clean command with --zero switch", style='warn')
            to_screen("to have track of previously downloaded subtitles.", style='warn')
            to_screen("\tezsub clean --all --zero", style='ok')
        to_screen("\r\nAre you sure to delete all previously downladed subtitles? (y/N): ", style='red;bold', end='')
        answer = input('')
        if answer.lower() not in ['y', 'yes']:
            to_screen("\r\nCancelled.", style="info")
            return None
        results, selected = cache.all_titles()
    elif req.exact:
        results, selected = cache.exact_search(req.exact)
    else:  # use title
        results = cache.search(req.title)
        selected = select(results, req.auto_select)

    if not selected:
        raise NoResultError

    paths = [results[s-1]['path'] for s in selected]
    lngs = parse_lngs(req.lngs)
    to_clean = prune(paths, lngs)
    if not to_clean:
        raise NothingToCleanError

    files = [item['path'] for item in to_clean]

    size_before = get_size(cache.subtitles)
    if req.zero:
        action = cache.zero(files)
    else:
        action = cache.delete(files)

    size_after = get_size(cache.subtitles)
    to_screen(f"\n{human_readable(size_before-size_after)} freed by {action} files.\n", style='warn')

    return None



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import os
import shutil
import zipfile

from ezsub import const
from ezsub.errors import CacheIsEmptyError
from ezsub.utils import get_title, to_screen


class Cache(object):
    def __init__(self, root=const.ROOT):
        self.root = root
        self.subtitles = root.joinpath('subtitles')
        self.subtitles.mkdir(parents=True, exist_ok=True)
        emptied = True
        while emptied:
            emptied = self.delete_empty_folders()

    def search(self, title):
        to_screen("[cache] ", end='')
        to_screen(f"{self.subtitles}", style="info")
        if self.is_empty(self.subtitles):
            raise CacheIsEmptyError

        max_score = 1
        titles = list()
        for child in self.subtitles.iterdir():
            score = self._get_match_score(title, child.name)
            item = {
                'path': child.resolve(),
                'title': get_title(child)
            }
            if score > max_score:
                max_score = score
                titles = [item, ]
            elif score == max_score:
                titles.append(item)
        return titles

    def exact_search(self, title):
        return (
            [{'path': self.subtitles.joinpath(title).resolve(), 'title': ''}],  # results
            [1, ]   # selected
        )

    def all_titles(self):
        results = [{'path': child.resolve(), 'title': get_title(child)} for child in self.subtitles.iterdir()]
        selected = list(range(1, 1+len(results)))
        return results, selected

    def exists(self, path):
        return self.root.joinpath(path).exists()

    def get_child(self, child, mk_parents=False):
        if str(child).startswith('/'):
            child = f".{child}"
        path = self.root.joinpath(child)
        if mk_parents:
            path.parent.mkdir(parents=True, exist_ok=True)
        return path.resolve()

    def delete_empty_folders(self, root=None):
        emptied = False
        if not root:
            root = self.subtitles
        for child in root.iterdir():
            if child.is_dir():
                if self.is_empty(child):
                    child.rmdir()
                    emptied = True
                else:
                    emptied = emptied or self.delete_empty_folders(child)
        return emptied

    @staticmethod
    def _get_match_score(title, target):
        # TODO: add partial word score
        title_words = set(title.lower().replace('+', ' ').replace('-', ' ').split())
        score = 0
        for word in title_words:
            if str(target).__contains__(word):
                score += 1
        return score

    @staticmethod
    def empty_zipfile(path):
        with zipfile.ZipFile(path, 'w'):
            pass
        return None

    def zero(self, files):
        for file in files:
            self.empty_zipfile(file)
        return 'emptying'

    @staticmethod
    def delete(files):
        for file in files:
            file.unlink()
        return 'deleting'

    @staticmethod
    def is_empty(path):
        if path.is_dir():
            return not os.listdir(path)
        return False


def filter_langs(path, lngs):
    final_paths = list()
    for language in lngs.values():
        p = path.joinpath(language)
        if p.exists():
            final_paths.append(p)
    return final_paths


def count_each_language(result, lngs):
    splitted = {lng: 0 for lng in lngs.keys()}
    mapper = {language: lng for lng, language in lngs.items()}
    for file in result:
        language = file['path'].parent.name
        splitted[mapper[language]] += 1
    # remove empty languages
    splitted = {key: value for key, value in splitted.items() if value}
    return splitted


def prune(paths, lngs):
    pruned = []
    for path in paths:
        folders = filter_langs(path, lngs)
        result = [
            {'url': '', 'path': child}
            for folder in folders
            for child in folder.iterdir()
        ]
        splitted = count_each_language(result, lngs)
        to_screen("\n    path: ", end='')
        to_screen(f"{path.resolve()}", style="info")
        to_screen("   found: ", end='')
        to_screen(splitted, style="ok")
        pruned = pruned[:] + result[:]
    splitted = count_each_language(pruned, lngs)
    to_screen("\ntotal: ", end='')
    to_screen(splitted, style="ok")
    return pruned



#!/usr/bin/env python3
# -*- coding: utf-8 -*-


class InvalidSiteError(Exception):
    pass


class NoResultError(Exception):
    pass


class LoginFailedError(Exception):
    pass


class NothingToCleanError(Exception):
    pass


class NothingToDownloadError(Exception):
    pass


class NothingToExtractError(Exception):
    pass


class EmptyArchiveError(Exception):
    pass


class BadCompressedFileError(Exception):
    pass


class JobDone(Exception):
    pass


class WrongLineNumberError(Exception):
    pass


class CacheIsEmptyError(Exception):
    pass


class NoSiteIsAvailableError(Exception):
    pass


class GetContentFailed(Exception):
    pass


class NetworkError(Exception):
    pass

class ForciblyClosedError(Exception):
    pass



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from ezsub import const
from ezsub.conf import UserConf
from ezsub.mirrors import Mirror
from ezsub.utils import to_screen
from ezsub.errors import LoginFailedError


def login():
    """This function is for subscene only. Other mirrors do not have login page"""
    timeout = 3 * const.TIMEOUT
    site = Mirror('subscene')
    site.select_first_responding(timeout)
    to_screen("It takes some seconds. In success, it will display new token.")
    try:
        token = site.login(timeout)
        to_screen(f"token:", end='')
        to_screen(token, style='ok')
        configs = UserConf()
        configs.set_captcha(token)
        to_screen("saved new token to user.conf", style='ok')
    except LoginFailedError:
        to_screen("Login failed.", style='red')
    return None



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from ezsub import const
from ezsub.utils import to_screen
from ezsub.cli.arguments import CliArgs
from ezsub.errors import JobDone, WrongLineNumberError


def history(argv):
    req = CliArgs(argv)
    if req.command not in ['history', 'h']:
        add_to_history(argv)
        return req
    else:
        if not req.subcommand or req.subcommand == 'show':
            show_history()
        elif req.subcommand == 'run':
            return get_from_history(req.line)
    return None


def add_to_history(argv):
    command = ' '.join(argv) + '\n'
    path = const.HISTORY
    if not path.exists():
        with open(path, 'w') as file:
            file.writelines(command)
    else:
        with open(path, 'r') as file:
            commands = file.readlines()
        if not (commands and command == commands[0]):  # if history is clear or command is new
            commands.insert(0, command)
        with open(path, 'w') as file:
            file.writelines(commands)
    return None


def show_history():
    with open(const.HISTORY, 'r') as file:
        lines = file.readlines()
    for i, line in enumerate(lines):
        to_screen(f"{i+1}: {const.PROGRAMNAME} {line[:-1]}")  # last character is new line.
    raise JobDone


def get_from_history(linenumber):
    with open(const.HISTORY, 'r') as file:
        commands = file.readlines()
    if linenumber <= len(commands):
        command = commands[linenumber-1]
        return CliArgs(command.split())
    else:
        raise WrongLineNumberError



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import logging
import subprocess

from ezsub import const

logger = logging.getLogger(__name__)

s = const.Style()


def to_screen(msg='', style=None, silent=False, flush=True, **kwargs):
    if not silent:
        if style:
            msg = s.render(style, msg)
        print(str(msg), flush=flush, **kwargs)
    return None


def full2abbr(language):
    for key, value in const.LANGUAGE_PAIRS.items():
        if value == language:
            return key
    else:
        raise ValueError(f"language '{language}' is not supported.")


def abbr2full(lng):
    return const.LANGUAGE_PAIRS[lng]


def is_valid_lng(lng):
    return lng in const.SUPPORTED_LNGS


def parse_lngs(lngs_string):
    lngs = lngs_string.split()
    parsed = dict()
    if "*" in lngs:
        return const.LANGUAGE_PAIRS
    for lng in lngs:
        if is_valid_lng(lng):
            parsed[lng] = abbr2full(lng)
        else:
            logger.warn(f"'{lng}' is not a valid language abbr. Ignored.")
            to_screen(f"'{lng}' is not a valid language abbr. Ignored.", style="warn")
    return parsed or {'en': 'english'}


def is_valid_integer(i):
    try:
        int(i)
        return True
    except ValueError:
        return False


def filter_valid_choice(text, maximum):
    numbers = text.split(',')
    numbers = [int(n) for n in numbers if is_valid_integer(n)]
    return list(set([n for n in numbers if (0 < n < 1+maximum)]))


def get_user_choice(results):
    mx = len(results)
    if mx == 1:
        text = "  Press Enter to select only result"
    else:
        text = "  Enter title numbers, comma separated numbers"

    while True:
        try:
            to_screen(f"{text} [1]: ", end='')
            selected = input() or '1'
            answer = filter_valid_choice(selected, mx)
            if answer:
                return answer
            else:
                raise IndexError
        except IndexError:
            to_screen("    Oops! not valid. Try again.", style="warning")
    return None


def show_to_select(results):
    to_screen("--------------------------------------------")
    max_width = len(str(len(results)))
    for i, res in enumerate(results):
        to_screen(f"  {str(i+1).rjust(max_width, ' ')} - {res['title']}")
    to_screen("---------------------------------------------")
    return None


def select(results, auto_select):
    selected = []
    if results:
        show_to_select(results)
        if auto_select:
            to_screen('  select: auto select first result')
            selected = [1, ]
        else:
            selected = get_user_choice(results)
    return selected


def windows_size(path):
    command = [
        'powershell',
        '-noprofile',
        '-command',
        f"Get-ChildItem -File -Recurse {path} | Measure-Object -Sum -Property Length | Select-Object -ExpandProperty Sum"
    ]
    size = subprocess.check_output(command)
    if size:
        return int(size)
    else:
        return 0
    


def unix_size(path):
    command = ['du', '-s', '-B 1', path]
    size = subprocess.check_output(command)
    return int(size.split()[0].decode('utf-8'))


def human_readable(size):
    for unit in ['', 'K', 'M', 'G', 'T']:
        if size < 1024.0:
            return f"{size:03.2f} {unit}B"
        size = size / 1024.0


def machine_readable(size):
    units = {"B": 1, "KB": 1024, "MB": 1024**2, "GB": 1024**3, "TB": 1024**4}
    number, unit = size.split()
    return int(float(number)*units[unit])


def get_size(path, form='machine'):
    if const.OS == "Windows":
        size = windows_size(path)
    else:
        size = unix_size(path)
    if form == 'human':
        return human_readable(size)  # string
    elif form == 'machine':
        return size  # int
    return None


def get_title(path):
    return " ".join(path.name.split('-')).title()


def count_children(path, count, generation=1):
    '''
    generation 1 means direct children only
    generetion 2 means children of children only - children are excluded.
    generetion 3 ....
    '''
    if generation == 1:
        return count + len([child for child in path.iterdir()])
    else:
        for child in path.iterdir():
            count = count_children(child, count, generation-1)
    return count



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from ezsub.conf import UserConf, display_configs


def config(req):
    configs = UserConf()
    if not req.subcommand or req.subcommand == 'show':
        display_configs(configs.path)
    elif req.subcommand == 'set':
        section, option = req.option.split('.')
        configs.set_option(section, option, req.value)
    return None



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import sys
import logging
import traceback
from datetime import datetime
from urllib.parse import urlparse

import requests
from bs4 import BeautifulSoup
from requests_futures.sessions import FuturesSession

from ezsub import const
from ezsub.conf import UserConf
from ezsub.utils import to_screen
from ezsub.errors import (
    LoginFailedError,
    NoSiteIsAvailableError,
    GetContentFailed,
    NoResultError,
    ForciblyClosedError,
    NetworkError
)

cur = const.Curser
logger = logging.getLogger(__name__)
headers = { 'User-Agent': 'Mozilla/5.0'}
MDB = {
    'subscene': {
        "base_url": "https://subscene.com",
        "query_path": "/subtitles/searchbytitle",
        "method": requests.post,
        "login_path": '/account/login',
        "captcha": UserConf().get_captcha(),
        "selectors": {
            "title": "li div[class='title'] a",
            "link": "tr td[class='a1'] a",
            "btn": "a[id='downloadButton']"
        }
    },
    'hasti': {
        "base_url": "http://mysub.fun",
        "query_path": "/subtitles/searchbytitle",
        "method": requests.get,
        "login_path": '',
        "captcha": '',
        "selectors": {
            "title": "li div[class='title'] a",
            "link": "tr td[class='a1'] a",
            "btn": "#downloadButton",
            "release": "li.release div",
            "author": "li.author a",
            "date": "#details ul li"
        }
    },
    'subf2m': {
        "base_url": "https://subf2m.co",
        "query_path": "/subtitles/searchbytitle",
        "method": requests.get,
        "login_path": '',
        "captcha": '',
        "selectors": {
            "title": "li div[class='title'] a",
            "link": "a[class='download icon-download']",
            "btn": "a[id='downloadButton']"
        }
    },
    'delta':
    {
        "base_url": "https://sub.deltaleech.com",
        "query_path": "/subtitles/searchbytitle",
        "method": requests.post,
        "login_path": '',
        "captcha": '',
        "selectors": {
            "title": "li div[class='title'] a",
            "link": "tr td[class='a1'] a",
            "btn": "a[id='downloadButton']"
        }
    }
}

def available(pagetext):
    for sign in const.SIGNS:
        if sign.lower() in str(pagetext).lower():
            # to_screen('\n[Warning] Temporary unavailable. Try again later or check the site in browser.')
            return False
    for sign in const.BAD:
        if sign.lower() in str(pagetext).lower():
            to_screen("\nBad request. maybe captcha is expired. login again with 'ezsub login'", style="warn")
            return False
    return True


class Mirror(object):
    def __init__(self, names=const.SITE):
        '''names is a space separated site names in preferred order.'''
        self.names = names.split()

    def select_first_responding(self, timeout=const.TIMEOUT, ignore=''):
        not_checked = const.MIRRORS[:]
        for name in self.names:
            if name == ignore:
                not_checked.remove(name)
                continue
            self._fill_attributes(name)
            if self._is_responding(timeout):
                break
            else:
                not_checked.remove(self.name)
        else:
            # if still there are sites left to try
            if not_checked:
                to_screen("trying other mirrors...", style="yellow;italic")
            for name in not_checked:
                self._fill_attributes(name)
                if self._is_responding(timeout):
                    break
            else:
                raise NoSiteIsAvailableError
        return None

    def get_sub_details(self, page_text):
        soup = BeautifulSoup(page_text, 'html.parser')
        btn = soup.select_one(self.selectors['btn'])
        if btn:
            return {'download_link': btn['href']}
        return False

    def search(self, title):
        path = self.query_path
        data = None
        if self.method == requests.get:
            path += f"?query={title}"
        elif self.method == requests.post:
            data = {
                'query': title,
                'l': '',
                'g-recaptcha-response': self.captcha
            }
        to_screen("  query: ", end='')
        to_screen(f"{self.base_url}{path}", style="italic;info")
        page_text = self._get_page_text(path, data=data)
        retry = const.RETRY
        while (not page_text) and retry:
            to_screen("current mirror is not responding. trying other mirrors...", style="warn")
            self.select_first_responding(ignore=self.name)
            self._get_page_text(path, data)
            retry = retry - 1
        if not page_text:
            raise GetContentFailed
        soup = BeautifulSoup(page_text, 'html.parser')
        titles = soup.select(self.selectors['title'])
        aggregated = {title.attrs['href']: title.text for title in titles}
        return [{'path': p, 'title': t} for p, t in aggregated.items()]

    @staticmethod
    def exact_search(title):
        return (
            [{'path': f"/subtitles/{title}", 'title': ''}], # results
            [1,]   # selected
        )

    def get_subs(self, path):
        page_text = self._get_page_text(path)
        soup = BeautifulSoup(page_text, 'html.parser')
        subs = soup.select(self.selectors['link'])
        return {sub['href'] for sub in subs}

    def _get_page_text(self, path, data=None, timeout=const.TIMEOUT, retry=const.RETRY):
        while retry:
            try:
                page = self.method(self.base_url + path, data=data, headers=headers, timeout=timeout)
                page.encoding = 'utf-8'
                return page.text
            except Exception as e:
                to_screen("getting page content failed. for more info see log.", style="warn")
                logger.warn(e)
                logger.debug(traceback.format_exc())
            retry = retry - 1
            timeout = 2 * timeout  # double timeout
            if retry:
                to_screen("retry with timeout doubled...", style="warn")
        raise GetContentFailed

    def _is_responding(self, timeout=const.TIMEOUT):
        try:
            to_screen(f"checking '{self.name}': ", end='')
            to_screen(f"{self.base_url}/", style="info", end='')
            to_screen(" is ", end='')
            r = requests.head(self.base_url, headers=headers, timeout=timeout)
            if r.status_code == requests.codes['ok']:
                to_screen('OK', style="bold;ok")
                return True
            else:
                to_screen(f"not responding (status: {r.status_code})", style="warn")
        except Exception as e:
            to_screen("failed with error. see log. trying next mirror...", style="warn")
            logger.warn(e)
            logger.debug(traceback.format_exc())
        return False

    def mass_request(self, paths):
        session = FuturesSession(max_workers=const.MAX_WORKERS)
        n = len(paths)
        no_links = []
        to_download = []
        requests = [session.get(self.base_url + path, headers=headers) for path in paths]
        for i, path in enumerate(paths):
            try:
                page_text = requests[i].result().text
            except ConnectionResetError as e:
                logger.warn(e)
                logger.debug(traceback.format_exc())
                raise ForciblyClosedError
            except Exception as e:
                logger.warn(e)
                logger.debug(traceback.format_exc())
                raise NetworkError
            link = self.get_sub_details(page_text)
            url = self.base_url + path
            to_screen(f"\rgetting subtitles info... {i+1}/{n}", end='')  # progress
            if link:
                item = {
                    'path': path,
                    'url': url,
                    'dlink': link['download_link']
                }
                to_download.append(item)
            else:  # no link is found
                no_links.append(url)
        else:
            to_screen("\rgetting subtitles info... ", end='')
            to_screen(f"{cur.CFH}done!", style="ok")

        if no_links:
            to_screen("\nno download link is found for these urls:", style="warning")
            for link in no_links:
                to_screen(f'       {link}', style="info")
        return to_download

    def mass_download(self, to_download):
        session = FuturesSession(max_workers=const.MAX_WORKERS)
        all_requests = [session.get(self.base_url + sub['dlink'], headers=headers) for sub in to_download]
        n = len(to_download)
        to_extract = []
        for i, subtitle in enumerate(to_download):
            file = subtitle['path']
            to_screen(f"\rdownloading... {i+1}/{n}", end='')  # progress
            try:
                file_content = all_requests[i].result().content
            except ConnectionResetError as e:
                logger.warn(e)
                logger.debug(traceback.format_exc())
                raise ForciblyClosedError
            except Exception as e:
                logger.warn(e)
                logger.debug(traceback.format_exc())
                raise NetworkError

            with open(file, "w+b") as f:                
                f.write(file_content)
            to_extract.append(subtitle)
        else:
            to_screen("\rdownloading... ", end='')
            to_screen(f"{cur.CFH}done!", style="ok")
        return to_extract

    def login(self, timeout=const.TIMEOUT):
        session = requests.Session()
        try:
            session.get(self.base_url + self.login_path, headers=headers, timeout=timeout)
            return session.cookies.get_dict()['idsrv.xsrf']
        except:
            raise LoginFailedError

    def _fill_attributes(self, name):
        if name not in const.MIRRORS:
            to_screen(f"\r[site] ignoring invalid site '{name}' and using '{const.SITE}' instead.", style="warning")
            name = const.SITE
        self.name = name
        self.base_url = MDB[name]['base_url']
        self.query_path = MDB[name]['query_path']
        self.method = MDB[name]['method']
        self.login_path = MDB[name]['login_path']
        self.captcha = MDB[name]['captcha']
        self.selectors = MDB[name]['selectors']


def get_soup(url):
    session = FuturesSession(max_workers=const.MAX_WORKERS)
    req = session.get(url, headers=headers)
    r = req.result()
    r.encoding = 'utf-8'
    return BeautifulSoup(r.text, 'html.parser')


def get_subtitle_info(soup, selectors, url):
    '''get soup of subtitle download page and returns info'''
    info = dict()
    info["url"] = url
    info["language"] = url.split('/')[-2]
    info["author"] = soup.select_one(selectors.author).text.strip()
    releases = soup.select(selectors.release)
    info["releases"] = [div.text.strip() for div in releases]
    info['download'] = soup.select_one(selectors.btn)['href']
    upload_date = " ".join(soup.select_one(selectors.date).text.split()[1:4])
    info["date"] = datetime.strptime(upload_date, '%m/%d/%Y %I:%M %p')
    return info


def get_all_subtitle_urls(soup):
    '''get soup of title page or filtered language and returns all subtitles download page url'''
    subs = soup.select("tbody tr td[class='a1'] a")
    return {a['href'] for a in subs}


def get_available_languages(soup):
    '''get soup of a title page and returns all available languages for that title'''
    language_rows = soup.select('tbody tr td[colspan="5"]')
    return [row['id'] for row in language_rows if row.has_attr('id')]





#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from pathlib import Path

from ezsub import const
from ezsub.cache import Cache
from ezsub.mirrors import Mirror
from ezsub.destination import Destination
from ezsub.utils import to_screen, select, parse_lngs
from ezsub.errors import NoResultError, NothingToDownloadError, NetworkError


cur = const.Curser


def download(req):
    site = Mirror(req.site)
    site.select_first_responding()
    cache = Cache()
    destination = Destination(req.destination, req.group, req.open_after)

    if req.exact:
        results, selected = site.exact_search(req.exact)
    else:  # use title
        results = site.search(req.title)
        selected = select(results, req.auto_select)

    if not selected:
        raise NoResultError

    paths = [results[s-1]['path'] for s in selected]

    lngs = parse_lngs(req.lngs)
    new_subs = prune(paths, lngs, site, cache)
    if not new_subs:
        raise NothingToDownloadError

    if req.simulation:
        n = len(new_subs)
        for i, path in enumerate(new_subs):
            file = cache.get_child(f'{path}.zip', mk_parents=True)
            to_screen(f"\rcreating empty zip files... {i+1}/{n}", end='')  # progress stats
            cache.empty_zipfile(file)
        else:
            to_screen("\rcreating empty zip files... ", end='')
            to_screen(f"{cur.CFH}done", style='ok')
    else:
        try:
            to_download = site.mass_request(new_subs)
        except Exception as error:
            raise error
        for index, item in enumerate(to_download):
            to_download[index]['path'] = cache.get_child(f"{item['path']}.zip", mk_parents=True)
        to_extract = site.mass_download(to_download)
        destination.extract(to_extract)
    return None


def prune(paths, lngs, site, cache):
    pruned = []
    for path in paths:
        to_screen("\n     url: ", end='')
        to_screen(f"{site.base_url}{path}", style="info")
        links = site.get_subs(path)
        results = [
            link
            for link in links
            if link.split('/')[-2] in lngs.values()  # filter language
        ]
        splitted = count_each_language(results, lngs)
        to_screen("     all: ", end='')
        to_screen(splitted, style="ok")
        new_subs = [
            path
            for path in results
            if not cache.exists(f'.{path}.zip')  # filter already downloaded
        ]
        splitted = count_each_language(new_subs, lngs)
        to_screen("     new: ", end='')
        to_screen(splitted, style="ok")
        pruned = pruned[:] + new_subs[:]

    splitted = count_each_language(pruned, lngs)
    to_screen("\ntotal: ", end='')
    to_screen(splitted, style="ok")
    return pruned


def count_each_language(results, lngs):
    splitted = {lng: 0 for lng in lngs.keys()}
    mapper = {language: lng for lng, language in lngs.items()}
    for link in results:
        language = link.split('/')[-2]
        splitted[mapper[language]] += 1
    # remove empty languages
    splitted = {key: value for key, value in splitted.items() if value}
    return splitted



#!/usr/bin/env python3
# coding: utf-8

from setuptools import setup

# package information (is filling from ezsub/const.py)
__version__ = ''
__author__ = ''
__contact__ = ''
__url__ = ''
__license__ = ''


# Get the information from ezsub/const.py without importing the package
exec(compile(open('ezsub/const.py').read(),
             'ezsub/const.py', 'exec'))

with open("README.md", encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='ezsub',
    version=__version__,
    description='Download subtitle from subscene',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url=__url__,
    author=__author__,
    author_email=__contact__,
    license=__license__,
    packages=['ezsub', 'ezsub.cli', 'ezsub.cli.commands'],
    entry_points={
        'console_scripts': ['ezsub = ezsub.cli:main'],
    },
    install_requires=[
        'bs4',
        'requests-futures',
        'rarfile'
    ],
    python_requires='>=3.7',
    zip_safe=False,
    keywords='subtitle movie subscene',
    include_package_data=True,
    classifiers=[
        'Programming Language :: Python :: 3 :: Only',
        'Development Status :: 5 - Production/Stable',
        'Operating System :: POSIX :: Linux',
        'Environment :: Console',
        'Intended Audience :: End Users/Desktop',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Topic :: Utilities'
    ],
    project_urls={
        'source': __url__
    }
)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import sys
import logging
import argparse

from ezsub import const
from ezsub.utils import to_screen
from ezsub.cli.commands import (
    download, config, update, login, backup, history, info, clean, unzip
)
from ezsub.errors import (
    JobDone,
    WrongLineNumberError,
    NothingToCleanError,
    NothingToDownloadError,
    NothingToExtractError,
    NoResultError,
    CacheIsEmptyError,
    NoSiteIsAvailableError,
    GetContentFailed,
    ForciblyClosedError,
    NetworkError
)


def get_user_loglevel():
    parser = argparse.ArgumentParser(add_help=False)
    parser.add_argument("--loglevel")
    args, _ = parser.parse_known_args(sys.argv[1:])
    loglevel = args.loglevel
    if loglevel:
        numeric_level = getattr(logging, loglevel.upper(), None)
        if isinstance(numeric_level, int):
            to_screen(f"loglevel is set to {loglevel.upper()}", style="ok")
            return loglevel.upper()
        else:
            to_screen(f"Invalid log level {loglevel}. Ignored.", style="warn")
    return ''


logging.basicConfig(
    filename=const.LOGFILE,
    filemode=const.LOGFILEMODE,
    level=get_user_loglevel() or const.LOGLEVEL,
    format=const.LOGFORMAT
)
logger = logging.getLogger()


def main():
    try:
        req = history(sys.argv[1:])
        if req.command not in ['update', 'u']:
            update(just_remind=True)

        if req.command in ['dl', 'd', 'download']:
            download(req)
        elif req.command in ['unzip', 'x']:
            unzip(req)
        elif req.command in ['config', 'cfg']:
            config(req)
        elif req.command in ['update', 'u']:
            update()
        elif req.command in ['login', 'l']:
            login()
        elif req.command in ['backup', 'b']:
            backup(req)
        elif req.command in ['info', 'i']:
            info(req)
        elif req.command in ['clean']:
            clean(req)
    except KeyboardInterrupt:
        to_screen("\nTerminated by user.", style="red;bold")
    except NothingToCleanError:
        to_screen("\nNothing to clean.", style="warn;bold")
    except NothingToExtractError:
        to_screen("\nNothing to extract.", style="warn;bold")
    except NothingToDownloadError:
        to_screen("\nNothing to download.", style="warn;bold")
    except NoResultError:
        to_screen("\nNo Result for this title.", style="warn;bold")
    except CacheIsEmptyError:
        to_screen("\nCache folder is empty.", style="warn;bold")
    except NoSiteIsAvailableError:
        to_screen("\nSites are not accessible. check internet connection.", style="red;bold")
    except GetContentFailed:
        to_screen("\nGetting page content is failed. try later.", style="red")
    except ForciblyClosedError:
        to_screen("\nServer closed connections forcibly :(", style="red")
    except NetworkError:
        to_screen("\nNetwork error. See log for more details.", style="red")
    except JobDone:
        pass
    except WrongLineNumberError:
        to_screen("\nWrong line number", style="warn;bold")
    to_screen('\a')
    sys.exit(0)


if __name__ == "__main__":
    main()
